@@ -183,9 +183,9 @@ _dump_trap_stats(netsnmp_session *sess)
 #endif /* NETSNMP_NO_TRAP_STATS */
 
 int
-netsnmp_add_notification_session(netsnmp_session * ss, int pdutype,
-                                 int confirm, int version, const char *name,
-                                 const char *tag, const char* profile)
+netsnmp_add_closable_notification_session(netsnmp_session *ss, int close_sess, int pdutype,
+                                          int confirm, int version, const char *name,
+                                          const char *tag, const char* profile)
 {
     if (NETSNMP_RUNTIME_PROTOCOL_SKIP(version)) {
         DEBUGMSGTL(("trap", "skipping trap sink (version 0x%02x disabled)\n",
@@ -201,6 +201,7 @@ netsnmp_add_notification_session(netsnmp_session * ss, int pdutype,
         struct agent_add_trap_args args;
         DEBUGMSGTL(("trap", "adding callback trap sink (%p)\n", ss));
         args.ss = ss;
+        args.close_sess = close_sess;
         args.confirm = confirm;
         args.nameData = name;
         args.nameLen = (NULL == name) ? 0 : strlen(name);
@@ -234,6 +235,19 @@ netsnmp_add_notification_session(netsnmp_session * ss, int pdutype,
     return 1;
 }
 
+/* This is a wrapper to netsnmp_add_closable_notification_session
+   for backward compatibility
+ */
+int
+netsnmp_add_notification_session(netsnmp_session *ss, int pdutype,
+                                 int confirm, int version, const char *name,
+                                 const char *tag, const char* profile)
+{
+    return netsnmp_add_closable_notification_session(ss, 1, pdutype, confirm,
+                                                     version, name,
+                                                     tag, profile);
+}
+
 /*
  * xxx needs update to support embedded NUL.
  * xxx should probably also be using and unregister callback, similar to
@@ -260,12 +274,36 @@ netsnmp_unregister_notification(const char *name, u_char len)
                          "netsnmp_unregister_notification not supported\n"));
 }
 
+int
+handle_disconnect_packet(int operation, netsnmp_session *session, int reqid,
+                            netsnmp_pdu *pdu, void *magic)
+{
+    if (NETSNMP_CALLBACK_OP_DISCONNECT == operation) {
+        netsnmp_transport *t = snmp_sess_transport(snmp_sess_pointer(session));
+        char *addr_string = NULL;
+
+        if (t && (addr_string = netsnmp_transport_peer_string(t, t->remote, t->remote_length))) {
+            snmp_log(LOG_WARNING,
+                        "send_trap: session closed by snmptrap (%s)\n", addr_string);
+            free(addr_string);
+        } else {
+            snmp_log(LOG_WARNING,
+                        "send_trap: session %8p closed by snmptrap\n", session);
+        }
+        snmp_log(LOG_WARNING,
+                    "send_trap: re-opening of TCP connection is not supported.\n");
+
+        netsnmp_unregister_notification(session->paramName, strlen(session->paramName));
+    }
+    return 1;
+}
+
 int
 add_trap_session(netsnmp_session * ss, int pdutype, int confirm,
                          int version)
 {
-    return netsnmp_add_notification_session(ss, pdutype, confirm, version,
-                                            NULL, NULL, NULL);
+    return netsnmp_add_closable_notification_session(ss, 0, pdutype, confirm, version,
+                                                     NULL, NULL, NULL);
 }
 
 #ifndef NETSNMP_FEATURE_REMOVE_REMOVE_TRAP_SESSION
@@ -326,9 +364,10 @@ netsnmp_create_v1v2_notification_session(const char *sink, const char* sinkport,
     }
 
     snmp_sess_init(&session);
+    session.callback = handle_disconnect_packet;
     session.version = version;
     if (com) {
-        session.community = (u_char *) NETSNMP_REMOVE_CONST(char *, com);
+        session.community = (u_char *)strdup(com);
         session.community_len = strlen(com);
     }
 
@@ -357,7 +396,7 @@ netsnmp_create_v1v2_notification_session(const char *sink, const char* sinkport,
             client_addr = "localhost";
         tspec.source = client_addr;
     }
-    session.localname = NETSNMP_REMOVE_CONST(char *,tspec.source);
+    session.localname = tspec.source ? strdup(tspec.source) : NULL;
 
     tspec.application = "snmptrap";
     if (NULL == sinkport)
@@ -374,13 +413,18 @@ netsnmp_create_v1v2_notification_session(const char *sink, const char* sinkport,
         /** diagnose snmp_open errors with the input netsnmp_session pointer */
         snmp_sess_perror("snmpd: netsnmp_create_notification_session",
                          &session);
+        netsnmp_cleanup_session(&session);
         /* transport freed by snmp_add */
         return NULL;
     }
 
-    rc = netsnmp_add_notification_session(sesp, pdutype,
-                                          (pdutype == SNMP_MSG_INFORM),
-                                          version, name, tag, profile);
+    netsnmp_cleanup_session(&session);
+
+    rc = netsnmp_add_closable_notification_session(sesp,
+                                                   !(t->flags & NETSNMP_TRANSPORT_FLAG_STREAM),
+                                                   pdutype,
+                                                   (pdutype == SNMP_MSG_INFORM),
+                                                   version, name, tag, profile);
     if (0 == rc)
         return NULL;
 
@@ -1564,15 +1608,17 @@ netsnmp_create_v3user_notification_session(const char *dest, const char *user,
 
     snmp_sess_init(&session);
 
+    session.callback = handle_disconnect_packet;
+
     session.version = SNMP_VERSION_3;
 
-    session.peername = NETSNMP_REMOVE_CONST(char*,dest);
+    session.peername = strdup(dest);
 
-    session.securityName = NETSNMP_REMOVE_CONST(char*,user);
+    session.securityName = strdup(user);
     session.securityNameLen = strlen(user);
 
-    if (NULL != context) {
-        session.contextName = NETSNMP_REMOVE_CONST(char*,context);
+    if (context) {
+        session.contextName = strdup(context);
         session.contextNameLen = strlen(context);
     }
 
@@ -1648,21 +1694,20 @@ netsnmp_create_v3user_notification_session(const char *dest, const char *user,
         goto bail;
     }
 
-    if (netsnmp_add_notification_session(ss, pdutype,
-                                         (pdutype == SNMP_MSG_INFORM),
-                                         ss->version, notif_name, notif_tag,
-                                         notif_profile) != 1) {
+    if (netsnmp_add_closable_notification_session(ss,
+                                                  !(transport->flags & NETSNMP_TRANSPORT_FLAG_STREAM),
+                                                  pdutype,
+                                                  (pdutype == SNMP_MSG_INFORM),
+                                                  ss->version, notif_name, notif_tag,
+                                                  notif_profile) != 1) {
         DEBUGMSGTL(("trap:v3user_notif_sess", "add notification failed\n"));
         snmp_close(ss);
-        ss = NULL;
-        goto bail;
+        return NULL;
     }
 
   bail:
     /** free any allocated mem in session */
-    SNMP_FREE(session.securityAuthProto);
-    SNMP_FREE(session.securityPrivProto);
-    SNMP_FREE(session.contextEngineID);
+    netsnmp_cleanup_session(&session);
 
     return ss;
 }
@@ -1755,6 +1800,7 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
             free(argv[argn - 1]);
         goto cleanup;
     }
+    session.callback = handle_disconnect_packet;
     ss = snmp_add(&session, transport, NULL, NULL);
     for (; argn > 0; argn--)
         free(argv[argn - 1]);
@@ -1786,9 +1832,11 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
                                 NETSNMP_DS_LIB_DISABLE_V1))
         traptype = SNMP_MSG_TRAP;
 #endif
-    netsnmp_add_notification_session(ss, traptype,
-                                     (traptype == SNMP_MSG_INFORM),
-                                     ss->version, name, tag, profile);
+    netsnmp_add_closable_notification_session(ss,
+                                              !(transport->flags & NETSNMP_TRANSPORT_FLAG_STREAM),
+                                              traptype,
+                                              (traptype == SNMP_MSG_INFORM),
+                                              ss->version, name, tag, profile);
 
   cleanup:
     netsnmp_cleanup_session(&session);
