@@ -6,31 +6,31 @@
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-features.h>
 
-#if HAVE_SYS_PARAM_H
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #include <sys/stat.h>
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
 # endif
 #endif
-#if HAVE_DIRENT_H
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #else
 # define dirent direct
-# if HAVE_SYS_NDIR_H
+# ifdef HAVE_SYS_NDIR_H
 #  include <sys/ndir.h>
 # endif
-# if HAVE_SYS_DIR_H
+# ifdef HAVE_SYS_DIR_H
 #  include <sys/dir.h>
 # endif
-# if HAVE_NDIR_H
+# ifdef HAVE_NDIR_H
 #  include <ndir.h>
 # endif
 #endif
@@ -60,7 +60,7 @@
 #endif
 #endif
 
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
@@ -102,7 +102,7 @@ netsnmp_feature_require(date_n_time);
  *      de_p                    swi->swi_dep
  */
 typedef struct {
-#if HAVE_LIBRPM
+#ifdef HAVE_LIBRPM
     char           *swi_directory;
 #else
     const char     *swi_directory;
@@ -110,7 +110,7 @@ typedef struct {
     char            swi_name[SNMP_MAXPATH];     /* XXX longest file name */
     int             swi_index;
 
-#if HAVE_LIBRPM
+#ifdef HAVE_LIBRPM
     const char     *swi_dbpath;
 
     time_t          swi_timestamp;      /* modify time on database */
@@ -229,8 +229,14 @@ init_hr_swinst(void)
         snprintf(path, sizeof(path), "%s/Packages", swi->swi_dbpath);
         if (stat(path, &stat_buf) == -1)
             snprintf(path, sizeof(path), "%s/packages.rpm", swi->swi_dbpath);
+        /* check for SQLite DB backend */
+        if (stat(path, &stat_buf) == -1)
+            snprintf(path, sizeof(path), "%s/rpmdb.sqlite", swi->swi_dbpath);
         path[ sizeof(path)-1 ] = 0;
         swi->swi_directory = strdup(path);
+#ifdef HAVE_RPMGETPATH
+        rpmFreeRpmrc();
+#endif
     }
 #else
 #  ifdef _PATH_HRSW_directory
@@ -412,7 +418,7 @@ var_hrswinst(struct variable * vp,
             else
                 long_return = 0;        /* predates this agent */
         } else
-#if NETSNMP_NO_DUMMY_VALUES
+#ifdef NETSNMP_NO_DUMMY_VALUES
             return NULL;
 #else
             long_return = 363136200;
@@ -516,7 +522,7 @@ var_hrswinst(struct variable * vp,
                     goto err;
             } else {
 err:
-#if NETSNMP_NO_DUMMY_VALUES
+#ifdef NETSNMP_NO_DUMMY_VALUES
                 ret = NULL;
 #else
                 sprintf(string, "back in the mists of time");
